import { CommonModule } from '@angular/common';
import { Injectable, Component, Input, EventEmitter, NgModule, Output, ComponentFactoryResolver, ViewChild, ViewContainerRef, defineInjectable } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AngularTreeGridService {
    constructor() { }
    /**
     * @param {?} data
     * @param {?} configs
     * @return {?}
     */
    findTopParentNode(data, configs) {
        /** @type {?} */
        const ids = data.map((/**
         * @param {?} element
         * @return {?}
         */
        element => element[configs.id_field]));
        /** @type {?} */
        let top_parents = [];
        // Find parents ie if parent_id is not present in ids.
        data.forEach((/**
         * @param {?} element
         * @return {?}
         */
        element => {
            if (!ids.includes(element[configs.parent_id_field])) {
                top_parents.push(element[configs.parent_id_field]);
            }
        }));
        // Remove duplicates
        top_parents = top_parents.filter((/**
         * @param {?} item
         * @param {?} pos
         * @param {?} self
         * @return {?}
         */
        function (item, pos, self) {
            return self.indexOf(item) === pos;
        }));
        return top_parents;
    }
    /**
     * @param {?} data
     * @param {?} processed_data
     * @param {?} expand_tracker
     * @param {?} configs
     * @return {?}
     */
    processData(data, processed_data, expand_tracker, configs) {
        /** @type {?} */
        const top_parents = this.findTopParentNode(data, configs);
        data.map((/**
         * @param {?} rec
         * @return {?}
         */
        rec => {
            rec.pathx = [];
            rec.leaf = false;
        }));
        top_parents.forEach((/**
         * @param {?} top_parent
         * @return {?}
         */
        top_parent => {
            /** @type {?} */
            const children = this.findChildren(data, top_parent, configs);
            this.orderData(data, processed_data, configs, children, [], 0);
        }));
        processed_data.map((/**
         * @param {?} rec
         * @return {?}
         */
        rec => {
            /** @type {?} */
            const parent_pathx = rec.parent_pathx;
            rec.parent_pathx = parent_pathx.join('.');
            parent_pathx.push(rec[configs.id_field]);
            // Add current id to create current path.
            rec.pathx = parent_pathx.join('.');
            expand_tracker[rec.pathx] = false;
        }));
        // Expand root.
        expand_tracker[''] = true;
    }
    /**
     * @param {?} data
     * @param {?} id
     * @param {?} configs
     * @return {?}
     */
    findChildren(data, id, configs) {
        return data.filter((/**
         * @param {?} rec
         * @return {?}
         */
        rec => rec[configs.parent_id_field] === id));
    }
    /**
     * @param {?} data
     * @param {?} processed_data
     * @param {?} configs
     * @param {?} parents
     * @param {?} paths
     * @param {?} levelx
     * @return {?}
     */
    orderData(data, processed_data, configs, parents, paths, levelx) {
        parents.forEach((/**
         * @param {?} parent
         * @return {?}
         */
        parent => {
            /** @type {?} */
            const children = this.findChildren(data, parent[configs.id_field], configs);
            if (children.length === 0) {
                parent.leaf = true;
                parent.levelx = levelx;
                parent.parent_pathx = [...paths];
                processed_data.push(parent);
            }
            else {
                parent.parent_pathx = [...paths];
                parent.levelx = levelx;
                processed_data.push(parent);
                /** @type {?} */
                const newPaths = [...paths, parent[configs.id_field]];
                this.orderData(data, processed_data, configs, children, newPaths, levelx + 1);
            }
        }));
    }
}
AngularTreeGridService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
AngularTreeGridService.ctorParameters = () => [];
/** @nocollapse */ AngularTreeGridService.ngInjectableDef = defineInjectable({ factory: function AngularTreeGridService_Factory() { return new AngularTreeGridService(); }, token: AngularTreeGridService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AngularTreeGridComponent {
    /**
     * @param {?} angularTreeGridService
     */
    constructor(angularTreeGridService) {
        this.angularTreeGridService = angularTreeGridService;
        this.processed_data = [];
        this.expand_tracker = {};
        this.columns = [];
        this.default_configs = {
            css: {
                expand_class: 'plus',
                collapse_class: 'minus',
                add_class: 'plus',
                edit_class: '',
                delete_class: '',
                save_class: '',
                cancel_class: ''
            },
            data_loading_text: 'Loading...',
            row_class_function: (/**
             * @return {?}
             */
            () => true)
        };
        this.default_column_config = {
            sorted: 0,
            sort_type: null,
            editable: false,
            hidden: false,
            sortable: true
        };
        this.cellclick = new EventEmitter();
        this.expand = new EventEmitter();
        this.collapse = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        this.setDefaultConfigs();
        this.setColumnNames();
        this.angularTreeGridService.processData(this.data, this.processed_data, this.expand_tracker, this.configs);
    }
    /**
     * @return {?}
     */
    setDefaultConfigs() {
        this.processed_data = [];
        this.configs = Object.assign({}, this.default_configs, this.configs);
        if (!this.configs.id_field) {
            window.console.error('id field is mandatory!');
        }
        if (!this.configs.parent_id_field) {
            window.console.error('parent_id field is mandatory!');
        }
    }
    /**
     * @return {?}
     */
    setColumnNames() {
        this.columns = this.configs.columns ? this.configs.columns : [];
        // If columns doesn't exist in user's object.
        if (!this.configs.columns) {
            /** @type {?} */
            const column_keys = Object.keys(this.data[0]);
            // Insert Header and default configuration.
            column_keys.forEach((/**
             * @param {?} key
             * @return {?}
             */
            key => {
                this.columns.push(Object.assign({ 'header': key, 'name': key }, this.default_column_config));
            }));
        }
        else {
            // Insert Header and default configuration.
            for (let i = 0; i < this.columns.length; i++) {
                this.columns[i] = Object.assign({}, this.default_column_config, this.columns[i]);
            }
        }
    }
}
AngularTreeGridComponent.decorators = [
    { type: Component, args: [{
                selector: 'db-angular-tree-grid',
                template: "<table class=\"db-tree-view\">\r\n    <thead \r\n        db-tree-head\r\n        [processed_data]=\"processed_data\"\r\n        [expand_tracker]=\"expand_tracker\"\r\n        [data]=\"data\"\r\n        [columns]=\"columns\"\r\n        [configs]=\"configs\">\r\n    </thead>\r\n\r\n    <tbody \r\n        db-tree-body\r\n        [processed_data]=\"processed_data\"\r\n        [expand_tracker]=\"expand_tracker\"\r\n        [data]=\"data\"\r\n        [columns]=\"columns\"\r\n        [configs]=\"configs\"\r\n        [cellclick]=\"cellclick\"\r\n        [expand]=\"expand\"\r\n        [collapse]=\"collapse\"\r\n    >        \r\n    </tbody>\r\n</table>",
                styles: [".db-tree-view{line-height:1.5em;border-collapse:collapse;border-spacing:0;display:table;width:100%;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all;border:1px solid gray;color:#4b4b4b}"]
            }] }
];
/** @nocollapse */
AngularTreeGridComponent.ctorParameters = () => [
    { type: AngularTreeGridService }
];
AngularTreeGridComponent.propDecorators = {
    data: [{ type: Input }],
    configs: [{ type: Input }],
    cellclick: [{ type: Output }],
    expand: [{ type: Output }],
    collapse: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TreeBodyComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onRowExpand(event) {
        /** @type {?} */
        const row_data = event.data;
        if (this.expand_tracker[row_data.pathx]) {
            this.collapse.emit(event);
        }
        else {
            this.expand.emit(event);
        }
        this.expand_tracker[row_data.pathx] = !this.expand_tracker[row_data.pathx];
    }
}
TreeBodyComponent.decorators = [
    { type: Component, args: [{
                selector: '[db-tree-body]',
                template: "<tr *ngFor=\"let rec of processed_data\">\n  <ng-container *ngIf=\"expand_tracker[rec.parent_pathx]\">\n    <td *ngFor=\"let column of columns; index as i\" [ngClass]=\"{'column-hide': column.hidden}\">\n      <db-tree-cell\n        [configs]=\"configs\"\n        [column]=\"column\"\n        [index]=\"i\"\n        [row_data]=\"rec\"\n        [expand_tracker]=\"expand_tracker\"\n        [cellclick]=\"cellclick\"\n        (rowexpand)=\"onRowExpand($event)\"\n      >\n      </db-tree-cell>\n    </td>\n  </ng-container>\n</tr>",
                styles: [".db-tree-view{line-height:1.5em;border-collapse:collapse;border-spacing:0;display:table;width:100%;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all;border:1px solid gray;color:#4b4b4b}tr.selected{background-color:#e2e7eb}tr span.parent_container{padding-left:45px}tr.child{background:#fff}tr.child td:nth-child(2){padding:.875rem 1.25rem .875rem 2.5rem!important}tr.parent{background:#fafbff}tr td{border:1px solid #cdd5dc;vertical-align:middle;position:relative;padding:.5rem}tr td.column-hide{display:none}tr td.clear-left-border{border-left:0!important}tr td.clear-right-border{border-right:0!important}"]
            }] }
];
/** @nocollapse */
TreeBodyComponent.ctorParameters = () => [];
TreeBodyComponent.propDecorators = {
    data: [{ type: Input }],
    configs: [{ type: Input }],
    processed_data: [{ type: Input }],
    expand_tracker: [{ type: Input }],
    columns: [{ type: Input }],
    cellclick: [{ type: Input }],
    expand: [{ type: Input }],
    collapse: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TreeCellComponent {
    constructor() {
        this.rowexpand = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.is_expand_column = this.index === 0;
        this.show_expand_icon = !this.row_data.leaf;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onCellClick(event) {
        if (this.index === 0 && !this.row_data.leaf) {
            this.rowexpand.emit({ event: event, data: this.row_data });
        }
        this.cellclick.emit({ event: event, data: this.row_data });
    }
}
TreeCellComponent.decorators = [
    { type: Component, args: [{
                selector: 'db-tree-cell',
                template: "<div (click)=\"onCellClick($event)\">\n  <ng-container *ngIf=\"is_expand_column\">\n    <div [ngStyle]=\"{'padding-left': row_data.leaf ? row_data.levelx * 20 + 'px' : row_data.levelx * 10 + 'px'}\">\n      <ng-container *ngIf=\"!expand_tracker[row_data.pathx]\">\n          <i \n          [ngClass]=\"configs.css.expand_class\" \n          class=\"expand-icon\" \n          aria-hidden=\"true\"\n          *ngIf=\"show_expand_icon\"\n        ></i>\n      </ng-container>\n      <ng-container *ngIf=\"expand_tracker[row_data.pathx]\">\n          <i \n          [ngClass]=\"configs.css.collapse_class\" \n          class=\"expand-icon\" \n          aria-hidden=\"true\"\n          *ngIf=\"show_expand_icon\"\n        ></i>\n      </ng-container>      \n      <ng-container [ngSwitch]=\"column.type\">\n        <db-custom-cell-component \n          *ngSwitchCase=\"'custom'\"\n          [column]=\"column\" \n          [row_data]=\"row_data\">\n        </db-custom-cell-component>\n        <db-tree-cell-view\n          *ngSwitchDefault \n          [column] = \"column\"\n          [row_data] = \"row_data\">\n        </db-tree-cell-view>\n      </ng-container>\n    </div>\n  </ng-container>\n  <ng-container *ngIf=\"!is_expand_column\" [ngSwitch]=\"column.type\">\n    <db-custom-cell-component \n      *ngSwitchCase=\"'custom'\"\n      [column]=\"column\" \n      [row_data]=\"row_data\">\n    </db-custom-cell-component>\n    <db-tree-cell-view\n      *ngSwitchDefault \n      [column] = \"column\"\n      [row_data] = \"row_data\">\n    </db-tree-cell-view>\n  </ng-container>  \n</div>\n",
                styles: [".db-tree-view{line-height:1.5em;border-collapse:collapse;border-spacing:0;display:table;width:100%;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all;border:1px solid gray;color:#4b4b4b}i.expand-icon{margin-right:33px}i.plus{border:1px solid transparent}i.plus:before{content:'';width:18px;height:1px;border-top:2px solid #222;display:block;position:absolute;margin-top:-15px;margin-left:6px}i.plus:after{content:'';width:1px;height:18px;border-right:2px solid #222;display:block;position:absolute;margin-top:-23px;margin-left:14px}i.minus{border:1px solid transparent}i.minus:before{content:'';width:18px;height:1px;border-top:2px solid #222;display:block;position:absolute;margin-top:-15px;margin-left:6px}"]
            }] }
];
/** @nocollapse */
TreeCellComponent.ctorParameters = () => [];
TreeCellComponent.propDecorators = {
    configs: [{ type: Input }],
    index: [{ type: Input }],
    row_data: [{ type: Input }],
    column: [{ type: Input }],
    expand_tracker: [{ type: Input }],
    cellclick: [{ type: Input }],
    rowexpand: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TreeCellViewComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
}
TreeCellViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'db-tree-cell-view',
                template: "<span>{{column.renderer ? column.renderer(row_data[column.name], row_data) : row_data[column.name]}}</span>\n",
                styles: [""]
            }] }
];
/** @nocollapse */
TreeCellViewComponent.ctorParameters = () => [];
TreeCellViewComponent.propDecorators = {
    column: [{ type: Input }],
    row_data: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CustomCellViewComponent {
    /**
     * @param {?} resolver
     */
    constructor(resolver) {
        this.resolver = resolver;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.column.component && !this.customComponent) {
            this.createCustomComponent();
            this.callOnComponentInit();
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.customComponent) {
            this.customComponent.destroy();
        }
    }
    /**
     * @protected
     * @return {?}
     */
    createCustomComponent() {
        /** @type {?} */
        const componentFactory = this.resolver.resolveComponentFactory(this.column.component);
        this.customComponent = this.dynamicTarget.createComponent(componentFactory);
    }
    /**
     * @protected
     * @return {?}
     */
    callOnComponentInit() {
        this.column.onComponentInit && this.column.onComponentInit(this.customComponent.instance);
        this.customComponent.instance.cell_value = this.row_data[this.column.name];
        this.customComponent.instance.row_data = this.row_data;
    }
}
CustomCellViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'db-custom-cell-component',
                template: `
      <ng-template #dynamicTarget></ng-template>
    `
            }] }
];
/** @nocollapse */
CustomCellViewComponent.ctorParameters = () => [
    { type: ComponentFactoryResolver }
];
CustomCellViewComponent.propDecorators = {
    column: [{ type: Input }],
    row_data: [{ type: Input }],
    dynamicTarget: [{ type: ViewChild, args: ['dynamicTarget', { read: ViewContainerRef },] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TreeCellModule {
}
TreeCellModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TreeCellComponent, TreeCellViewComponent, CustomCellViewComponent],
                imports: [
                    CommonModule
                ],
                exports: [TreeCellComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TreeBodyModule {
}
TreeBodyModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TreeBodyComponent],
                imports: [
                    CommonModule,
                    TreeCellModule
                ],
                exports: [TreeBodyComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TreeHeadComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
}
TreeHeadComponent.decorators = [
    { type: Component, args: [{
                selector: '[db-tree-head]',
                template: "<tr>\n  <th *ngFor=\"let column of columns\" [ngClass]=\"{'column-hide': column.hidden}\">\n    <div>{{column.header}}</div>    \n  </th>\n</tr>",
                styles: [".db-tree-view{line-height:1.5em;border-collapse:collapse;border-spacing:0;display:table;width:100%;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all;border:1px solid gray;color:#4b4b4b}tr{background:#fff}tr th{padding:.5rem;font-family:Exo;font-size:1rem;font-weight:600;line-height:1.25;color:#181818;border:1px solid #cdd5dc;vertical-align:middle;position:relative}tr th.column-hide{display:none}th.clear-left-border{border-left:0!important}th.clear-right-border{border-right:0!important}.column-hide{display:none}svg{width:25px;padding-right:5px}"]
            }] }
];
/** @nocollapse */
TreeHeadComponent.ctorParameters = () => [];
TreeHeadComponent.propDecorators = {
    data: [{ type: Input }],
    configs: [{ type: Input }],
    processed_data: [{ type: Input }],
    expand_tracker: [{ type: Input }],
    columns: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TreeHeadModule {
}
TreeHeadModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TreeHeadComponent],
                imports: [
                    CommonModule
                ],
                exports: [TreeHeadComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AngularTreeGridModule {
}
AngularTreeGridModule.decorators = [
    { type: NgModule, args: [{
                declarations: [AngularTreeGridComponent],
                imports: [
                    CommonModule,
                    TreeBodyModule,
                    TreeHeadModule
                ],
                exports: [AngularTreeGridComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { AngularTreeGridService, AngularTreeGridComponent, AngularTreeGridModule, TreeBodyComponent as ɵb, TreeBodyModule as ɵa, CustomCellViewComponent as ɵf, TreeCellViewComponent as ɵe, TreeCellComponent as ɵd, TreeCellModule as ɵc, TreeHeadComponent as ɵh, TreeHeadModule as ɵg };

//# sourceMappingURL=angular-tree-grid.js.map